package Simulate::ConwayLife;

# Simulate::ConwayLife

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA = qw(Exporter);
@EXPORT_OK  = qw( randomlife );

$VERSION = '0.50';


sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my (@board) = @_ if (@_);
	my $self = [@board];
	return bless $self, $class;
}

sub randomlife {
	# (columns, rows, percent chance of life in decimal (eg .8 = 80%))
	my ($y, $x, @board);
	my ($cols, $rows, $percent) = (shift, shift, shift);
	for $y (0 .. $rows-1) {
		for $x (0 .. $cols-1) {
			if (rand() < $percent) {
				$board[$y][$x] = 1;
			} else {
				$board[$y][$x] = 0;
			}
		}
	}
	return @board;
}

sub view_output {
	my $proto = shift;
	my $self  = ref($proto) && $proto;
	my ($y, $x, $n, $cols) = 0;
	my $output;
	my @board = @$self;
	for $y (0 .. $#board) {
		for $x (0 .. $#{$board[$y]}) {
			$output .= $board[$y][$x];
		}
		$output .= "\n";
	}
	return $output;
}

sub view_row {
	my $proto = shift;
	my $self  = ref($proto) && $proto;
	my $y = shift;
	my $x = 0;
	if ($y > $#{@$self}) {
		return 0;
	}
	my @out;
	for $x (0 .. $#{$$self[$y]}) {
		 push(@out, $$self[$y][$x]);
	}
	return @out;
}

sub view_col {
	my $proto = shift;
	my $self  = ref($proto) && $proto;
	my $x = shift;
	my $y = 0;
	if ($x > $#{$$self[$y]}) {
		return 0;
	}
	my @out;
	for $y (0 .. $#{@$self}) {
		 push(@out, $$self[$y][$x]);
	}
	return @out;
}

sub run {
	my $proto = shift;
	my $self  = ref($proto) && $proto;
	
	my $times = sprintf("%.0f", shift || 1);
	my @newboard;

	my ($i, $y, $x, $n, $cols, $surrounding) = 0;
	my $rows = $#{@$self};
	for ($i = 0; $i < $times; $i++) {
		for $y (0 .. $#{@$self}) {
			for $x (0 .. $#{@$self[$y]}) {
				$surrounding = 0;
				$cols = $#{@$self[$y]};
				if ( ($y-1) >= 0) { # north
					if ($$self[$y-1][$x]) {
						$surrounding++;
					}
				}
				if ( ($x+1) <= $cols && ($y-1) >= 0) { # north of east
					if ($$self[$y-1][$x+1]) {
						$surrounding++;
					}
				}

				if ( ($x+1) <= $cols) { # east
					if ($$self[$y][$x+1]) {
						$surrounding++;
					}
				}
				if ( ($x+1) <= $cols && ($y+1) <= $rows) { # south of east
					if ($$self[$y+1][$x+1]) {
						$surrounding++;
					}
				}
				if ( ($y+1) <= $#{@$self}) { # south
					if ($$self[$y+1][$x]) {
						$surrounding++;
					}
				}
				if ( ($x-1) >=0 && ($y+1) <= $rows ) { # south of west
					if ($$self[$y+1][$x-1]) {
						$surrounding++;
					}
				}

				if ( ($x-1) >= 0) { # west
					if ($$self[$y][$x-1]) {
						$surrounding++;
					}
				}
				if ( ($x-1) >= 0 && ($y-1) >= 0) { # north of west
					if ($$self[$y-1][$x-1]) {
						$surrounding++;
					}
				}
				if ($surrounding == 3) {
					$newboard[$y][$x] = 1;
				} elsif ($surrounding == 2) {
					$newboard[$y][$x] = $$self[$y][$x] ? 1 : 0;
				} else {
					$newboard[$y][$x] = 0;
				}
			}
		}

	}
	@$self = @newboard;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

Simulate::ConwayLife - Perl extension for simple life simulation.

=head1 SYNOPSIS

 use Simulate::ConwayLife;
 my $life = Simulate::ConwayLife->new( ([0, 0, 0],
                                  [1, 1, 1],
                                  [0, 0, 0]) );
 $life->run(1);
 print $life->view_output();

A 3x3 grid is created and run through the algorithm once, then the output is printed.

=item Methods

=item  randomlife();

 Returns an array with a random chance of life.

=over 3

 use Simulate::ConwayLife qw( randomlife );
 my $life = Simulate::ConwayLife->new(randomlife(5, 4, .4));
 $life->run(1); 
 print $life->view_output();

=back

A 5x4 grid with random life is created. There is a 40% chance of a 
cell having life. The grid is run once, then the output is printed. 


=item $life->view_output()
 Returns a string with the data in $life as ones and zeros.
 Line breaks separate each row.


=item $life->view_row(y)
 Returns an array of the data in the y row in $life.


=item $life->view_row(x)
 Returns an array of the data in the x column in $life. 



=head1 DESCRIPTION

Simulate::ConwayLife simulates life using John Conway's algorithm.
A grid of the cells is stored in a 2 dimensional array.

=head1 AUTHOR

Dan Bjorkegren, dan_b@mail.com

=cut